/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {Checkbox, Icon, Popup} from "semantic-ui-react";
import {SimpleVDivider} from "../../helpers/simpleVDivider";
import CommonHeaderBarContentWrapper from '../../commonHeaderBarContentWrapper'
import {
  justRunProgramDoExerciseAsync,
  runCompileSingleFileDoExercise,
  saveDoExerciseSolution,
  setShouldNotCountAsync,
  updateDoExerciseLockSolutionsFlagAsync
} from "../../../state/actions/doExerciseSite/doExerciseCrudActions";
import {
  JustRunProgramCommandForBackend, RunCompileSingleFileCommandForBackend, SolutionFileForTestServer
} from "../../../types/testsAndTestProtocol";
import {HelpPopup} from "../../helpers/helpPopup";
import {ReleaseDurationType} from "../../../types/ReleaseDurationType";
import {
  setIsJustRunProgramArgsInputDisplayedDoEditor,
  setIsLeftTimeStringDisplayed,
  setJustRunProgramArgsDoEditor,
  setSelectedConsoleViewTest
} from "../../../state/actions/doExerciseSite/doExerciseSiteActions";
import {addCloseWindowEvent, clearCloseWindowEvent} from "../../../helpers/eventsHelper";
import {Prompt} from "react-router";
import * as mousetrap from 'mousetrap'
import {notifySuccess} from "../../../helpers/notificationHelper";
import {leftIconPopupOffset} from "../../../constants";
import {askDialog, errorDialog} from "../../../helpers/dialogHelper";
import Logger from "../../../helpers/logger";
import {ProgramArgsInputView} from "../../helpers/programArgsInputView";
import {justRunProgramArgsMaxLength} from "../../../state/reducers/doExerciseSite/doExerciseSiteReducer";
import {MyPopup} from "../../helpers/myPopup";
import {getI18n, getRawI18n} from "../../../../i18n/i18nRoot";
import history from '../../../routerHistory'
import * as constants from '../../../constants'
import {saveDoExerciseAfterSolutionAsync} from '../../../state/actions/doExerciseSite/afterSolutionCrudActions'
import {SolutionFileDoExerciseFullBase} from '../../../types/exerciseSolution'
import {ErrorHelper} from '../../../helpers/errorHelper'
import {setIsLoggerPanelOpen} from '../../../state/actions/loggerPanelActions'

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
}

const mapStateToProps = (rootState: RootState /*, props: MyProps*/) => {
  return {
    //test0: rootState...
    //test: props.test

    releaseCode: rootState.doExerciseSiteState.exercise.exerciseRelease.releaseCode,
    doExercise: rootState.doExerciseSiteState.exercise,
    solution: rootState.doExerciseSiteState.solution,
    selectedTabIdDoEditor: rootState.doExerciseSiteState.doEditorState.selectedTabIdDoEditor,
    selectedTabIdDoAfterEditor: rootState.doExerciseSiteState.afterEditorState.selectedTabIdDoEditor,
    testTypes: rootState.testTypesState.testTypes,

    exercisePreview: rootState.doExerciseSiteState.exercisePreviewFrontendTemp,
    isLeftTimeStringDisplayed: rootState.doExerciseSiteState.isLeftTimeStringDisplayed,
    editorHasUnsavedChanges: rootState.doExerciseSiteState.doEditorState.hasUnsavedChanges,

    afterEditorHasUnsavedChanges: rootState.doExerciseSiteState.afterEditorState.hasUnsavedChanges,

    isJustRunProgramArgsInputDisplayed: rootState.doExerciseSiteState.isJustRunProgramArgsInputDisplayed,
    justRunProgramArgs: rootState.doExerciseSiteState.justRunProgramArgs,

    isNormalEditorSelected: rootState.doExerciseSiteState.topLeftSelectedTabIndex === 0,

    isLoading: rootState.doExerciseSiteState.isLoading
      || rootState.testTypesState.isLoading
      || rootState.pLangsState.isLoading
      || rootState.systemSettingsSiteState.isLoading,

    isSomeTestRunning: rootState.doExerciseSiteState.exercise.tests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.exercise.customTests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.afterNormalTests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.afterSubmitTests.some(p => p.uiIsRunning)
      || rootState.doExerciseSiteState.afterCustomTests.some(p => p.uiIsRunning)
    ,

    //can be simplified
    canUserEditExercise: (rootState.doExerciseSiteState.exercisePreviewFrontendTemp
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen)
      || (rootState.doExerciseSiteState.exercise.exerciseRelease.isReleased
        && rootState.doExerciseSiteState.exercise.exerciseRelease.hasLimitedWorkingTime === false
        && rootState.doExerciseSiteState.exercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Infinitely),

    isSomeDialogDisplayed: rootState.doExerciseSiteState.isCodeEditorSettingsDialogDisplayed
      || rootState.doExerciseSiteState.isCustomTestDialogDisplayed
      || rootState.doExerciseSiteState.isViewTestDialogDisplayed,

    afterSolution: rootState.doExerciseSiteState.afterSolution,


    //if this.props.exercisePreview.isStillOpen === false we ignore this flag because we then edit the after solution
    lockSolutionsFlagAndIsStillOpen: rootState.doExerciseSiteState.lockSolutionsFlag
      && rootState.doExerciseSiteState.exercisePreviewFrontendTemp !== null && rootState.doExerciseSiteState.exercisePreviewFrontendTemp.isStillOpen,
    lockSolutionsFlag: rootState.doExerciseSiteState.lockSolutionsFlag,


    firstName: rootState.userDataSettingsSate.userData.firstName,
    lastName: rootState.userDataSettingsSate.userData.lastName,
    isSiteHeaderBarNotDisplayed: rootState.siteHeaderBarState.isSiteHeaderBarDisplayedAndEnabled === false
      || rootState.siteHeaderBarState.isSiteHeaderBarCollapsed,

    isLoggingOut: rootState.siteHeaderBarState.isLoggingOut,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here

                                                                        saveDoExerciseSolution,
                                                                        runCompileSingleFileDoExercise,
                                                                        setIsLeftTimeStringDisplayed,
                                                                        setSelectedConsoleViewTest,
                                                                        setShouldNotCountAsync,
                                                                        justRunProgramDoExerciseAsync,
                                                                        setIsJustRunProgramArgsInputDisplayedDoEditor,
                                                                        setJustRunProgramArgsDoEditor,

                                                                        saveDoExerciseAfterSolutionAsync,
                                                                        updateDoExerciseLockSolutionsFlagAsync,
                                                                        setIsLoggerPanelOpen,

                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;


/**
 * if the left time is set and below or equals this the
 * icon will be different colored
 * @type {number}
 */
export const criticalLeftWorkingTimeInS = 60

class HeaderBar extends React.Component<Props, any> {

  htmlSaveButton: HTMLDivElement | undefined

  componentDidMount(): void {
    addCloseWindowEvent()

    mousetrap.bindGlobal(["ctrl+s", "meta+s"], this.simulateSaveButtonPress)

  }

  componentWillUnmount(): void {
    clearCloseWindowEvent()

    mousetrap.unbind(["ctrl+s", "meta+s"])
  }

  simulateSaveButtonPress = async (e: ExtendedKeyboardEvent | null) => {


    if (this.props.exercisePreview !== null && this.props.exercisePreview.isPermanentlyLocked) {
      Logger.log('exercise is permanently locked, cannot save')
      return
    }

    if (!this.htmlSaveButton) {
      Logger.log('htmlSaveButton was undefined')
      return
    }

    const editorTextArea = document.querySelector(':focus') as HTMLTextAreaElement

    if (!editorTextArea) {
      //then the editor is probably up to date...
      Logger.log('we assume the editor has no focus and thus is up to date')
    }

    if (editorTextArea)
      editorTextArea.blur() //this should blur the editor before saving... (and update the store files)

    await this.onSaveKeyPress(e)

    if (editorTextArea)
      editorTextArea.focus()
  }

  onSaveKeyPress = async (e: ExtendedKeyboardEvent | null) => {
    //else the browser would ask to save the site.. (on windows)

    if (e) {
      e.preventDefault()
    }

    if (this.props.exercisePreview !== null && this.props.exercisePreview.isPermanentlyLocked) {
      Logger.log('exercise is permanently locked, cannot save')
      return
    }

    if (this.props.isSomeDialogDisplayed) return

    if (this.props.isSomeTestRunning) return


    if (!this.props.canUserEditExercise) {

      if (!this.props.afterSolution) return
      //after solution is ready and can be edited

      const shouldResetAllTestResults = this.props.afterEditorHasUnsavedChanges

      const isOk = await this.props.saveDoExerciseAfterSolutionAsync(this.props.releaseCode,
                                                                     this.props.solution.pLangId,
                                                                     shouldResetAllTestResults,
                                                                     this.props.afterSolution, this.props.langId
      )
      if (isOk) {
        notifySuccess(getI18n(this.props.langId, 'Saved files'))
      } else {
        //error displayed in saveDoExerciseAfterSolutionAsync
      }
      return
    }

    if (this.props.lockSolutionsFlagAndIsStillOpen) return

    const shouldResetAllTestResults = this.props.editorHasUnsavedChanges

    const isOk = await this.props.saveDoExerciseSolution(this.props.releaseCode, shouldResetAllTestResults,
                                                         this.props.solution, this.props.langId
    )

    if (isOk) {
      notifySuccess(getI18n(this.props.langId, 'Saved files'))
    } else {
      //error displayed in saveDoExerciseSolution
    }

  }

  render(): JSX.Element {

    const allFiles =
      this.props.canUserEditExercise
      ?
      (this.props.solution && this.props.solution.solutionFiles
       ? this.props.solution.solutionFiles
       : [])
      : (
        this.props.afterSolution && this.props.afterSolution.solutionFiles
        ?
        this.props.afterSolution.solutionFiles
        : []
      )

    //this is not the real main file (saved in the solution) but the CURRENT FILE
    // noinspection TsLint
    const mainFile: SolutionFileDoExerciseFullBase | undefined = this.props.canUserEditExercise
                                                                 ? allFiles.find(
        p => p.id === this.props.selectedTabIdDoEditor)
                                                                 : allFiles.find(
        p => p.id === this.props.selectedTabIdDoAfterEditor)


    const isExerciseLocked = this.props.exercisePreview !== null && this.props.exercisePreview.isPermanentlyLocked

    const leftArea = (
      <div>

        <Prompt when={this.props.editorHasUnsavedChanges && !this.props.isLoggingOut}
                message={getI18n(this.props.langId, 'You have unsaved changes, do you really want to exit?')}/>

        {
          //save all files
          !this.props.isLoading &&
          <div className="flexed">
            <div ref={(r) => this.htmlSaveButton = r} className={
              ((this.props.canUserEditExercise || (!this.props.canUserEditExercise && !this.props.isNormalEditorSelected)) && !this.props.isSomeTestRunning) && (isExerciseLocked === false || (isExerciseLocked && !this.props.isNormalEditorSelected))
              ? 'clickable mar-right-half'
              : 'div-disabled mar-right-half'
            }
                 onClick={() => this.onSaveKeyPress(null)}
            >
              <Icon name="save"/>
              <span>
            {
              getI18n(this.props.langId, 'Save all files')
            }
          </span>
            </div>
            <HelpPopup defaultText={getI18n(this.props.langId,
                                            'Shortcut: ctrl+s/cmd+s. This will also clear all test results if the code was changed (because they are then invalid)'
            )}/>
            <SimpleVDivider/>
          </div>
        }

        {
          //compile single file
          !this.props.isLoading &&
          <div className="flexed">
            <div className={
              (allFiles.length > 0 && mainFile && ((this.props.canUserEditExercise || (!this.props.canUserEditExercise && !this.props.isNormalEditorSelected)) && !this.props.isSomeTestRunning) && (isExerciseLocked === false || (isExerciseLocked && !this.props.isNormalEditorSelected)))
              ? 'clickable mar-right-half'
              : 'div-disabled mar-right-half'
            }
                 onClick={async () => {

                   if (allFiles.length === 0) {
                     return
                   }

                   if (!mainFile) {
                     //...maybe the exercise has no default files...
                     //or no file is selected...
                     return
                   }

                   const command: RunCompileSingleFileCommandForBackend = {
                     generatedCode: this.props.releaseCode,
                     plangId: this.props.solution.pLangId,
                     mainFileName: mainFile.fileNameWithExtension,
                     files: allFiles.map<SolutionFileForTestServer>(p => {
                       return {
                         fileName: p.fileNameWithExtension,
                         fileContent: p.content
                       }
                     })
                   }

                   if (this.props.editorHasUnsavedChanges) {
                     await this.onSaveKeyPress(null)
                   }

                   //else the old result could be displayed...
                   //actually new test should be displayed (because in runCompileSingleFileDoExercise we forge a temp test)
                   //but this don't happen somehow so clear
                   this.props.setSelectedConsoleViewTest(null)

                   this.props.runCompileSingleFileDoExercise(command, this.props.testTypes)

                 }}
            >
              <Icon name="bug"/>
              <span>
            {
              getI18n(this.props.langId, 'Compile single file')
            }
          </span>
            </div>
            <HelpPopup
              defaultText={getI18n(this.props.langId,
                                   'Saves all files (ONLY if the editor has unsaved changes) and compiles the program with the current file as main file.'
              )}
              wide
            />
            <SimpleVDivider/>
          </div>
        }

        {
          //just run program
          //behind compile single file in case we want to make cmd args accessible inline
          !this.props.isLoading &&
          <div className="flexed">
            <div className={
              ((allFiles.length > 0 && (this.props.canUserEditExercise || (!this.props.canUserEditExercise && !this.props.isNormalEditorSelected)) && !this.props.isSomeTestRunning) && (isExerciseLocked === false || (isExerciseLocked && !this.props.isNormalEditorSelected)))
              ? 'clickable mar-right-half'
              : 'div-disabled mar-right-half'
            }
                 onClick={async () => {

                   if (allFiles.length === 0) {
                     return
                   }

                   const command: JustRunProgramCommandForBackend = {
                     CmdArgs: this.props.justRunProgramArgs
                              ? this.props.justRunProgramArgs
                              : '',
                     releaseCode: this.props.releaseCode,
                     plangId: this.props.solution.pLangId,
                   }

                   if (this.props.editorHasUnsavedChanges) {
                     await this.onSaveKeyPress(null)
                   }

                   //clear old result
                   this.props.setSelectedConsoleViewTest(null)

                   this.props.justRunProgramDoExerciseAsync(command, this.props.testTypes)

                 }}
            >
              <Icon name="rocket"/>
              <span>
                {
                  getI18n(this.props.langId, 'Just Run Program')
                }
              </span>
            </div>

            <HelpPopup className="mar-right-half"
                       defaultText={getI18n(this.props.langId,
                                            'Saves all files (ONLY if the editor has unsaved changes) and runs the program without any input (except the given args). If one arg should contain whitespaces enclose it in "" because args are separated by whitespaces. The max length for the program args is '
                       ) + justRunProgramArgsMaxLength}
                       wide
            />

            <div style={{display: 'inline-block'}} className="clickable mar-right-half" onClick={() => {
              this.props.setIsJustRunProgramArgsInputDisplayedDoEditor(
                !this.props.isJustRunProgramArgsInputDisplayed)
            }}>
              <Icon className={this.props.justRunProgramArgs
                               ? 'toggled-colored'
                               : ''}
                    name="font"/>
            </div>

            {
              this.props.isJustRunProgramArgsInputDisplayed &&
              <div>
                <ProgramArgsInputView programArgs={this.props.justRunProgramArgs}
                                      setProgramArgs={this.props.setJustRunProgramArgsDoEditor}/>
              </div>
            }


          </div>
        }

        {
          !this.props.doExercise.exerciseRelease.hideExerciseLeaveActions &&
          <SimpleVDivider/>
        }


        {
          !this.props.doExercise.exerciseRelease.hideExerciseLeaveActions &&
          <div className="clickable" onClick={() => {
            history.push(
              constants.openExercisesLinkPath)
          }}>
            <Icon name="reply"/>
            <span>
                {getI18n(this.props.langId, 'Leave')}
              </span>
          </div>
        }

      </div>
    )

    const isTimeLimited = this.props.doExercise.exerciseRelease.hasLimitedWorkingTime
      || (this.props.doExercise.exerciseRelease.releaseDurationType === ReleaseDurationType.Limited && !!this.props.doExercise.exerciseRelease.automaticEndAt)

    const isReleased = this.props.doExercise.exerciseRelease.isReleased

    const isShouldNotCountDisplayed = ((this.props.solution.shouldNotCount === false && this.props.exercisePreview && this.props.exercisePreview.isStillOpen))

    const isLeftTimeDisplayed = this.props.exercisePreview && isTimeLimited


    const rightArea = (
      <div className="flexed">

        {
          this.props.doExercise.exerciseRelease.showAdditionalLogButton &&
          <MyPopup
            trigger={
              <div className="clickable" onClick={() => {
                this.props.setIsLoggerPanelOpen(true)
              }}>
                <Icon className="mar-right-half" name="unordered list"/>
              </div>
            }
            content={getI18n(this.props.langId, "Displays the internal log")}
          />
        }
        {
          isShouldNotCountDisplayed &&
          (this.props.doExercise.exerciseRelease.showAdditionalLogButton) && //only display if we have an item on hte left side
          <SimpleVDivider/>
        }

        {
          //not yet checked
          isShouldNotCountDisplayed &&
          <div className="flexed mar-left">

            <div className={`mar-right-half ${isExerciseLocked ? 'div-disabled' : 'clickable'}`} onClick={async () => {

              const shouldNotCount = await askDialog(getI18n(this.props.langId, 'Participation should not count'),
                                                     getI18n(this.props.langId,
                                                             'Are you sure you want that your participation should not be counted? This cannot be undone!'
                                                     ), this.props.langId
              )

              if (!shouldNotCount) return

              try {
                await this.props.setShouldNotCountAsync(this.props.releaseCode)
              } catch (err) {
                ErrorHelper.makeServerErrorDialog(err, this.props.langId, 'Error',
                                                  "Could not set 'participation should not count'."
                )
              }

            }}>
              <Icon className="mar-right-half" name="flag outline"/>
              <span>
                  {
                    getI18n(this.props.langId, 'Should not count')
                  }
                </span>
            </div>

            <HelpPopup defaultText={getI18n(this.props.langId,
                                            'Your participation will not be counted (nevertheless a tutor might assess your submission, it\'s just an info for the tutors), this cannot be undone! This does not affect the left working time.'
            )}/>
          </div>
        }

        {
          isLeftTimeDisplayed &&
          (isShouldNotCountDisplayed || this.props.doExercise.exerciseRelease.showAdditionalLogButton) && //only display if we have an item on hte left side
          <SimpleVDivider/>
        }
        {
          isLeftTimeDisplayed &&
          <div className="flexed">
            <div>
              {
                this.props.exercisePreview.isStillOpen &&
                (this.props.exercisePreview.leftTimeInS === null
                  || (this.props.exercisePreview.leftTimeInS !== null && this.props.exercisePreview.leftTimeInS > criticalLeftWorkingTimeInS)) &&
                <MyPopup
                  trigger={
                    <div className="clickable"
                         onClick={() => {
                           this.props.setIsLeftTimeStringDisplayed(!this.props.isLeftTimeStringDisplayed)
                         }}
                    >
                      <Icon name="wait"/>
                    </div>
                  }
                  offset={leftIconPopupOffset}>
                  <Popup.Content>
                    <div>
                      <span dangerouslySetInnerHTML={getRawI18n(this.props.langId,
                                                                'Displays the remaining time. Click to show/hide. If the remaining time is <= '
                      )}></span>
                      <span>{`${criticalLeftWorkingTimeInS}${getI18n(this.props.langId, 's')}${getI18n(
                        this.props.langId,
                        ' the icon color will change.'
                      )}`}</span>
                    </div>
                  </Popup.Content>
                </MyPopup>
              }
              {
                this.props.exercisePreview.isStillOpen &&
                this.props.exercisePreview.leftTimeInS !== null &&
                this.props.exercisePreview.leftTimeInS <= criticalLeftWorkingTimeInS &&
                <MyPopup
                  trigger={
                    <div className="clickable"
                         onClick={() => {
                           this.props.setIsLeftTimeStringDisplayed(!this.props.isLeftTimeStringDisplayed)
                         }}
                    >
                      <Icon name="wait" className="orange-colored"/>
                    </div>
                  }
                  offset={leftIconPopupOffset}>
                  <Popup.Content>
                    <div>
                      <span dangerouslySetInnerHTML={getRawI18n(this.props.langId,
                                                                'Displays the remaining time. Click to show/hide. If the remaining time is <= '
                      )}></span>
                      <span>{`${criticalLeftWorkingTimeInS}${getI18n(this.props.langId, 's')}${getI18n(
                        this.props.langId,
                        ' the icon color will change.'
                      )}`}</span>
                    </div>
                  </Popup.Content>
                </MyPopup>
              }
              {
                !this.props.exercisePreview.isStillOpen &&
                <Icon name="wait" className="error-colored"/>
              }
            </div>

            <div>
              {
                this.props.isLeftTimeStringDisplayed &&
                this.props.exercisePreview.leftTimeString
              }
            </div>
          </div>
        }


        {
          //display the user name if the header bar is not visible
          this.props.isSiteHeaderBarNotDisplayed &&
          (isLeftTimeDisplayed || isShouldNotCountDisplayed || this.props.doExercise.exerciseRelease.showAdditionalLogButton) && //only display if we have an item on hte left side
          <SimpleVDivider/>
        }
        {
          this.props.isSiteHeaderBarNotDisplayed &&
          <div>
                <span className="mar-right-half" style={{opacity: 0.4}}>
                  <Icon name="user"/>
                </span>

            <span className="mar-right-half">
                  {this.props.firstName}
                </span>
            <span>
                  {this.props.lastName}
                </span>

          </div>
        }

        {
          isExerciseLocked &&
          (isLeftTimeDisplayed || isShouldNotCountDisplayed || this.props.doExercise.exerciseRelease.showAdditionalLogButton) && //only display if we have an item on hte left side
          <SimpleVDivider/>
        }

        {
          isExerciseLocked &&
          <div>
            <HelpPopup icon="lock" defaultText={getI18n(this.props.langId, 'Exercise is lock')}/>
          </div>
        }

      </div>
    )
    const rightAreaClosedRelease = (
      <div>
        {
          this.props.exercisePreview &&
          <div className="flexed">
            <div>
              {
                getI18n(this.props.langId, 'Release finished')
              }
            </div>

            {
              this.props.isSiteHeaderBarNotDisplayed &&
              <SimpleVDivider/>
            }
            {
              this.props.isSiteHeaderBarNotDisplayed &&
              <div>
                <span className="mar-right-half" style={{opacity: 0.4}}>
                  <Icon name="user"/>
                </span>

                <span className="mar-right-half">
                  {this.props.firstName}
                </span>
                <span>
                  {this.props.lastName}
                </span>

              </div>
            }

            {
              isExerciseLocked &&
              (this.props.exercisePreview || this.props.isSiteHeaderBarNotDisplayed) && //only display if we have an item on hte left side
              <SimpleVDivider/>
            }

            {
              isExerciseLocked &&
              <div>
                <HelpPopup icon="lock" defaultText={getI18n(this.props.langId, 'Exercise is lock')}/>
              </div>
            }

          </div>
        }
      </div>
    )


    return (
      <CommonHeaderBarContentWrapper leftArea={leftArea}
                                     rightArea={
                                       isReleased
                                       ?
                                       rightArea
                                       : rightAreaClosedRelease
                                     }
                                     disabled={this.props.isLoading || this.props.isSomeDialogDisplayed}
      />
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(HeaderBar)
