/**
 * Created by janis dähne.
 */

import * as React from "react";
import {connect} from "react-redux";
import {bindActionCreators, Dispatch} from "redux";
import {returntypeof} from 'react-redux-typescript';
import {RootState} from "../../../state/reducers";
import {SelectableUserFrontendExclusive} from "../../../types/users";
import {Table, Icon, Checkbox} from "semantic-ui-react";
import {CheckboxOnChangeEventData} from "../../../types/reactEvents";
import {deleteActivatedUserAsync} from "../../../state/actions/manageActivatedUsersSite/manageActivatedUsersSiteCrudActions";
import {
  setActivatedUserUiIsSelected,
  setAllActivatedUsersUiIsSelected, setIsChangeUserDataDialogDisplayed
} from "../../../state/actions/manageActivatedUsersSite/manageActivatedUsersSiteActions";
import {setTargetUser} from "../../../state/actions/manageActivatedUsersSite/dialogs/changeActivatedUserDataDialogActions";
import {copyToClipboard} from "../../../helpers/clipboard";
import {displayDateAndTime} from "../../../helpers/convertersAndTransformers";
import {askDialog} from "../../../helpers/dialogHelper";
import {HelpPopup} from "../../helpers/helpPopup";


import {getAssessmentStatisticsSiteToLinkPath} from "../../../constants";
import {Link} from "react-router-dom";
import {getI18n} from "../../../../i18n/i18nRoot";

//const css = require('./styles.styl');

export interface MyProps {
  //readonly test: string
  readonly activatedUser: SelectableUserFrontendExclusive
}

const mapStateToProps = (rootState: RootState, props: MyProps) => {
  return {
    //test0: rootState...
    //test: props.test
    activatedUser: props.activatedUser,
    userLangId: rootState.userDataSettingsSate.userData.settings
                ? rootState.userDataSettingsSate.userData.settings.langId
                : -1,

    systemRole: rootState.userDataSettingsSate.userData
                && rootState.userDataSettingsSate.userData.systemRole
                ? rootState.userDataSettingsSate.userData.systemRole
                : null,
    langId: rootState.i18nState.langId
  }
}

const mapDispatchToProps = (dispatch: Dispatch) => bindActionCreators({
                                                                        //imported reducer funcs here
                                                                        setIsChangeUserDataDialogDisplayed,
                                                                        setTargetUser,
                                                                        setActivatedUserUiIsSelected,
                                                                        setAllActivatedUsersUiIsSelected,
                                                                        deleteActivatedUserAsync,
                                                                      }, dispatch)


const stateProps = returntypeof(mapStateToProps);
const dispatchProps = returntypeof(mapDispatchToProps);
type Props = typeof stateProps & typeof dispatchProps;

//TODO docs beispiel
class testRow extends React.Component<Props, any> {
  render(): JSX.Element {

    const activatedUser = this.props.activatedUser

    return (
      <Table.Row>
        <Table.Cell>
          {
            (!activatedUser.isCurrentUser && (this.props.systemRole && this.props.systemRole.canDeleteActivatedUsers)) &&
            <Checkbox checked={activatedUser.uiIsSelected}
                      onChange={(event: any, data: CheckboxOnChangeEventData) => {
                        return this.props.setActivatedUserUiIsSelected(activatedUser.id, data.checked)
                      }}/>
          }

        </Table.Cell>

        <Table.Cell>
                      <span>
                        {activatedUser.lastName}
                      </span>
        </Table.Cell>
        <Table.Cell>
                      <span>
                        {activatedUser.firstName}
                      </span>
        </Table.Cell>
        <Table.Cell>
                      <span>
                        {activatedUser.token}
                      </span>
        </Table.Cell>
        <Table.Cell>
          <Icon name="copy" className="clickable"
                data-clipboard-text={activatedUser.email}
                onClick={() => {
                  copyToClipboard(activatedUser.email)
                }}
          />
          <span>
                        {activatedUser.email}
                      </span>
        </Table.Cell>
        <Table.Cell>
                      <span>
                        {
                          activatedUser.systemRole
                          ? activatedUser.systemRole.displayName
                          : getI18n(this.props.langId,
                                    'None'
                          )
                        }
                      </span>
        </Table.Cell>
        <Table.Cell>
                      <span>
                        {
                          displayDateAndTime(activatedUser.createdAt, this.props.userLangId)
                        }
                      </span>
        </Table.Cell>
        <Table.Cell>
                      <span>
                        {
                          displayDateAndTime(activatedUser.lastLoginAt, this.props.userLangId)
                        }
                      </span>
        </Table.Cell>
        <Table.Cell>
          <div>
            <Link style={{margin: '0'}} to={getAssessmentStatisticsSiteToLinkPath(activatedUser.id)}>
              <Icon name="area graph"/>
            </Link>
          </div>
        </Table.Cell>
        <Table.Cell>
          {
            this.props.systemRole &&
            (this.props.systemRole.canChangeOtherUsersSystemRole || this.props.systemRole.canChangeUserData)
            &&
            <div className="clickable" onClick={() => {
              this.props.setTargetUser(activatedUser)
              this.props.setIsChangeUserDataDialogDisplayed(true)
            }}>
              <Icon name="write"></Icon>
            </div>
          }

        </Table.Cell>
        <Table.Cell>
          {
            activatedUser.isCurrentUser &&
            <HelpPopup defaultText={getI18n(this.props.langId, 'This is you!')}/>
          }
          {
            (!activatedUser.isCurrentUser && (this.props.systemRole && this.props.systemRole.canDeleteActivatedUsers)) &&
            <div className="clickable"
                 onClick={async () => {

                   const shouldRemove = await askDialog(getI18n(this.props.langId, 'Delete activated user'),
                                                        `${getI18n(this.props.langId,
                                                                   'Are you sure you want to delete the activated user?'
                                                        )} (${activatedUser.lastName}, ${activatedUser.firstName})`, this.props.langId
                   )

                   if (shouldRemove) {
                     this.props.deleteActivatedUserAsync(activatedUser)
                   }

                 }}
            >
              <Icon name="remove"></Icon>
            </div>
          }


        </Table.Cell>
      </Table.Row>
    )
  }
}

export default connect(mapStateToProps, mapDispatchToProps)(testRow)
